#!/bin/bash
#
# (c) astx (at) aws-it.at, 2018 - 2019
# and (c) others - see DIYaudio thread https://www.diyaudio.com/forums/software-tools/331656-installing-using-ngspice-opensource-simulator.html
#
# $Id: step_opamps_loopgain.sh 1594 2019-02-08 12:10:41Z sta $
#
echo ""; echo ""
echo "#-----------------------------------------------------------------------"
echo "NOTE: For successful run you need at least ngspice 30 + pre-master fixes"
echo "After simulation run you can further explore the result data"
echo "Use 'quit' to exit simulation"
echo ""
echo "#-----------------------------------------------------------------------"
echo "If you want to run opamp stepping simulation now enter 'y' <ENTER>"
echo "#-----------------------------------------------------------------------"

fork_ngspice_process () {
	local mynet
	local opamp
	mynet=$1
	opamp=$2
	echo "*** fork ngspice run for device $opamp"
	# load schematic and create a fresh netlist
	gnetlist -g spice-sdb -o ${mynet}.${opamp}.net opamp_loopgain.sch
	# replace %MYOPA% from schematic by $opamp
	sed -i -r -e "s/\%MYOPA\%/${opamp}/g" ${mynet}.${opamp}.net
	ngspice -i ${mynet}.${opamp}.inc ${mynet}.${opamp}.net > tmp_log_${opamp}.data 2>&1
	echo "*** finished ngspice run for device $opamp"
}

read a
if [ "$a" == "y" ]; then

	# change this to your preferred editor. vi nano joe ...
	export SPICE_EDITOR="joe"

	numcpu=4
	if [ -f /proc/cpuinfo ]; then
		numcpu=$(cat /proc/cpuinfo |grep processor|wc -l);
		echo "found $numcpu processors"
	fi

	# temporary net list files - need uniqe name for parallel processing!
	mynet="tmp_opamp_loopgain"

	chead="* ng_script\n.control\n"
	cend="\n.endc\n"

	echo -en "${chead}" > tmp_load_result_files0.inc
	echo -en "${chead}" > tmp_load_result_files1.inc

	tianplots=''
	count=1

	for opamp in NE5532 NE5534-15P AD797-CMP AD8672 OP1177 LF412 LF412A AD745J $(cat adi_opamps.lst) $(cat adi1_opamps.lst)
	do
		opamp=$(echo $opamp|tr 'A-Z' 'a-z')
		ucopamp=$(echo $opamp|tr 'a-z' 'A-Z')

		# e.g. add/set some circuit params or options be4 simulation run
		echo -en "* ng_script\n*.options savecurrents\n.control\n\tsetcs opamp=\"${ucopamp}\"\n\techo \"OPA \$opamp\"\n.endc\n\n" > ${mynet}.${opamp}.inc

		# fork process into background
		fork_ngspice_process ${mynet} ${opamp} &

		# wait with loop continuation if too much processes are running
		while [ $(ps axf|grep "${mynet}" |grep -v grep|wc -l) -gt ${numcpu} ]
		do
			# echo -en '.'
			echo 'WAIT_FORKING'
			usleep 50000
		done
		echo "continue with loop"

		# generate load result files listing
		echo "load tmp_ac_${ucopamp}.data" >> tmp_load_result_files1.inc
		tianplots="${tianplots}ac${count} "
		count=$(($count+1))
	done

	usleep 50000
	# wait for all finished jobs
	echo "waiting for ngspice processes to finish"
	while [ $(ps axf|grep "${mynet}" |grep -v grep|wc -l) -gt 0 ]
	do
		echo -en '.'
		usleep 50000
	done
	echo " continue with results"

	echo -en "${cend}" >> tmp_load_result_files1.inc
	echo -en "set tianplots=( ${tianplots})${cend}" >> tmp_load_result_files0.inc

	# load (no netlist!) results only from stepping and display e.g. fft, fourier and input, output voltages
	# reset terminal if ngspice terminates unexpectedly
	ngspice opamp_loopgain_show.ngspice || reset
fi
